/*
    This program uses a shared variable to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads
    "alternating" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that each thread begins a computation exactly when the
    other threads ends a computation.

    This shows that the synchronization pattern implemented here
    is not all that complicated and does not really need an atomic
    operation like a semaphore.
*/
#include <stdio.h>
#include <windows.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = TRUE;

int sharedVariable;

int main( int argc, char *argv[])
{
   // thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // initialize the shared variable
   sharedVariable = 1;

   // create two threads
   printf("alternating1: Starting threads\n");
   CreateThread(NULL, 0, thread1, NULL, 0, NULL);
   CreateThread(NULL, 0, thread2, NULL, 0, NULL);

   Sleep(RUN_TIME*1000); // let the threads run for a while
   runFlag = FALSE;      // now stop the threads
   Sleep(200);           // give the threads time to stop

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   int i;
   int workTime;
   srand(GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      while( sharedVariable == 0 )
         Sleep(0);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sharedVariable = 0;
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   int i;
   int workTime;
   srand(1+GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      while( sharedVariable == 1 )
         Sleep(0);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      sharedVariable = 1;
   }
   return 0;
}//thread2
