/*
    This program uses two binary semaphores to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads
    "alternating" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that each thread begins a computation exactly when the
    other threads ends a computation.

    The wait-release pattern in this program is
             thread1    thread2
             -------    -------
              work       wait
              release    work
              wait       release
*/
#include <stdio.h>
#include <windows.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = TRUE;

HANDLE semaphore1;
HANDLE semaphore2;

int main( int argc, char *argv[])
{
   // thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // create two binary semaphores
   semaphore1 = CreateSemaphore(NULL, 0, 1, NULL); // not signaled
   semaphore2 = CreateSemaphore(NULL, 0, 1, NULL); // not signaled

   // create two threads
   printf("alternating3: Starting threads\n");
   CreateThread(NULL, 0, thread1, NULL, 0, NULL);
   CreateThread(NULL, 0, thread2, NULL, 0, NULL);

   Sleep(RUN_TIME*1000); // let the threads run for a while
   runFlag = FALSE;      // now stop the threads
   Sleep(200);           // give the threads time to stop

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   int i;
   int workTime;
   srand(GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      ReleaseSemaphore(semaphore2, 1, NULL);
      WaitForSingleObject(semaphore1, INFINITE);
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   int i;
   int workTime;
   srand(1+GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      WaitForSingleObject(semaphore2, INFINITE);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         Sleep(1000);
      }
      fprintf(stderr, "done");
      ReleaseSemaphore(semaphore1, 1, NULL);
   }
   return 0;
}//thread2
