/*
    This program uses two shared variables to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads in
    "lockstep" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that they begin each successive computation at the exact
    same time.

    This shows that the synchronization pattern implemented here
    is not all that complicated and does not really need an atomic
    operation like a semaphore. It needs only two shared variables.
*/
#include <stdio.h>
#include <windows.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = TRUE;

int sharedVariable1;
int sharedVariable2;

int main( int argc, char *argv[])
{
   // thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // initialize the shared variables
   sharedVariable1 = 0;
   sharedVariable2 = 0;

   // create two threads
   printf("lockstep1: Starting threads\n");
   CreateThread(NULL, 0, thread1, NULL, 0, NULL);
   CreateThread(NULL, 0, thread2, NULL, 0, NULL);

   Sleep(RUN_TIME*1000); // let the threads run for a while
   runFlag = FALSE;      // now stop the threads
   Sleep(20);            // give the threads time to stop

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   int workTime;
   srand(GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      sharedVariable2 = 1;
      while( sharedVariable1 == 0 )
         Sleep(0);
      sharedVariable1 = 0;
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread1 working for %d seconds\n", workTime);
      Sleep(workTime*1000);
      fprintf(stderr, "--done1\n");
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   int workTime;
   srand(1+GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      sharedVariable1 = 1;
      while( sharedVariable2 == 0 )
         Sleep(0);
      sharedVariable2 = 0;
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread2 working for %d seconds\n", workTime);
      Sleep(workTime*1000);
      fprintf(stderr, "--done2\n");
   }
   return 0;
}//thread2
