/*
    This program uses two binary semaphores to implement
    a simple synchronization pattern between two threads.
    The synchronization pattern is to keep the two threads in
    "lockstep" with each other. Each thread does a sequence
    of "computations," where each computation can take a
    different amount of time, and the threads are synchronized
    so that they begin each successive computation at the exact
    same time.

    Notice that this program has the same pattern of waits and
    releases as the program alternating2.c, the only difference
    being in the way the semaphores are initialized.

    What happens to this pattern if thread1 (or thread2) does one
    whole calculation before the other thread even starts (i.e., if
    one thread is started some time after the other thread)?

    The wait-release pattern in this program is
             thread1    thread2
             -------    -------
              wait       wait
              work       work
              release    release
*/
#include <stdio.h>
#include <windows.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = TRUE;

HANDLE semaphore1;
HANDLE semaphore2;

int main( int argc, char *argv[])
{
   // thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // create two binary semaphores
   semaphore1 = CreateSemaphore(NULL, 1, 1, NULL); // signaled
   semaphore2 = CreateSemaphore(NULL, 1, 1, NULL); // signaled

   // create two threads
   printf("lockstep2: Starting threads\n");
   CreateThread(NULL, 0, thread1, NULL, 0, NULL);
   CreateThread(NULL, 0, thread2, NULL, 0, NULL);

   Sleep(RUN_TIME*1000); // let the threads run for a while
   runFlag = FALSE;      // now stop the threads
   Sleep(20);            // give the threads time to stop

   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   int workTime;
   srand(GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      WaitForSingleObject(semaphore1, INFINITE);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread1 working for %d seconds\n", workTime);
      Sleep(workTime*1000);
      fprintf(stderr, "--done1\n");
      ReleaseSemaphore(semaphore2, 1, NULL);
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   int workTime;
   srand(1+GetTickCount());  // seed the random number generator
   while(runFlag)
   {
      WaitForSingleObject(semaphore2, INFINITE);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "Thread2 working for %d seconds\n", workTime);
      Sleep(workTime*1000);
      fprintf(stderr, "--done2\n");
      ReleaseSemaphore(semaphore1, 1, NULL);
   }
   return 0;
}//thread2
