/*
    This program uses two binary semaphores to keep two
    threads synchronized with each other. This kind of
    synchronization is used to make sure two threads
    start parts of a calcualtion at the same time.

    What would happen if the two threads were created in
    the signaled state?

    Can you get this kind of synchronization to work with
    3 threads? With N threads? (Hint: With N threads you
    need N(N-1) binaray semaphores. Can you do it with fewer
    counting semaphores?)
*/
#include <stdio.h>   // fprintf(), stderr
#include <stdlib.h>  // exit(), NULL
#include <unistd.h>  // sleep()
#include <pthread.h>

#define RUN_TIME 30  // run time in seconds

// global variables
int runFlag = 1;

pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t mutex2 = PTHREAD_MUTEX_INITIALIZER;

int main( int argc, char *argv[])
{
   void *thread1(void *vargp);  //thread function prototypes
   void *thread2(void *vargp);

   int threadParam1 = 0;  // thread function parameters
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // initialize the mutexes (in signaled state)
   //pthread_mutex_init(&mutex1, NULL);
   //pthread_mutex_init(&mutex2, NULL);

   // initially lock mutex2 so that thread1 can go first
   pthread_mutex_lock(&mutex1);
   pthread_mutex_lock(&mutex2);

   // create two threads
   printf("alternating: Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   sleep(RUN_TIME); // let the threads run for a while
   runFlag = 0;     // now stop the threads
   sleep(1);        // give the threads time to stop

   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread1(void *vargp)
{
   int i;
   int workTime;
   srand(1);      // seed the random number generator
   while(runFlag)
   {
      pthread_mutex_lock(&mutex1);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread1 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         sleep(1);
      }
      fprintf(stderr, "done");
      pthread_mutex_unlock(&mutex2);
   }
   return 0;
}//thread1


void *thread2(void *vargp)
{
   int i;
   int workTime;
   srand(2);      // seed the random number generator
   while(runFlag)
   {
      pthread_mutex_unlock(&mutex1);
      pthread_mutex_lock(&mutex2);
      workTime = 1 + rand()%9;  // 1 to 9 seconds of work time
      fprintf(stderr, "\nThread2 working for %d seconds", workTime);
      for (i = 0; i < workTime; i++)
      {
         fprintf(stderr, ".");
         sleep(1);
      }
      fprintf(stderr, "done");
   }
   return 0;
}//thread2
