/*
   This program uses something like a mutex to avoid the race condition.

   This program implements the mutex like object by using the Pentium's
   XCHG machine instruction. The mutex is implemented directly in this program.

   This version of the mutex is more efficient than the previous version
   because this version avoids a busy loop in the wait function.

   This program works correctly on multicore and multi-processor systems since
   the XCHG instruction implements the processors bus locking protocol.
*/
#include <stdio.h>
#include <windows.h>

// global variables
int runFlag = TRUE;
unsigned int count = 0, count1 = 0, count2 = 0;

int* mutex;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   // create a mutex (like) object
   mutex = (int*)malloc(sizeof(int));
   *mutex = 0;   // signaled

   // create threads
   printf("race5me: Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   // let the threads run for a while
   Sleep(2000);
   // now stop the threads
   runFlag = FALSE;
   Sleep(20);

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      //BeginCriticalSection()
      //{
again: __asm mov  eax, 1
      __asm mov  ebx, mutex
      __asm xchg eax, [ebx]
      __asm and  eax, eax
      __asm jz   done
      Sleep(0);  // we are blocked, so yield the processor
      __asm jmp again
done: __asm nop
      //}
      // enter critical section
      old_count = count++;
      new_count = count;
      //EndCriticalSection()
      //{
      *mutex = 0;
      //}

      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
      count1++;
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      //BeginCriticalSection()
      //{
again: __asm mov  eax, 1
      __asm mov  ebx, mutex
      __asm xchg eax, [ebx]
      __asm and  eax, eax
      __asm jz   done
      Sleep(0);
      __asm jmp again
done: __asm nop
      //}
      old_count = count++;
      new_count = count;
      //EndCriticalSection()
      //{
      *mutex = 0;
      //}

      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
      count2++;
   }
   return 0;
}//thread2
