/*
   This program uses something like a mutex to avoid the race condition.

   The definition of the mutex is in either the header file
   MyMutxHardware.h or in the header file MyMutexHardwareEfficient.h.

   The second header file defines a more efficient version of the
   mutex that avoids a continual busy loop in the wait function.
*/
#include <stdio.h>
#include <windows.h>

// Use a busy waiting version of the mutex:
//#include "MyMutexHardware.h"
// Or, use a more efficient (non-busy waiting) version of the mutex:
#include "MyMutexHardwareEfficient.h"

// global variables
int runFlag = TRUE;
unsigned int count = 0, count1 = 0, count2 = 0;

MyMutex mutex;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   // create a mutex (like) object
   mutex = CreateMyMutex(0); // signaled

   // create threads
   printf("race6m: Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   // let the threads run for a while
   Sleep(2000);
   // now stop the threads
   runFlag = FALSE;
   Sleep(20);

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      wait(mutex);
      old_count = count++;
      new_count = count;
      release(mutex);

      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
      count1++;
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      wait(mutex);
      old_count = count++;
      new_count = count;
      release(mutex);

      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
      count2++;
   }
   return 0;
}//thread2