/*
   This program uses software that acts kind of like a mutex
   to avoid the race condition.

   The code used in this program to implement mutual exclusion was
   invented by G.L. Peterson in 1981.

   This version of the code is more efficient than the last version
   because this version avoids the busy loop while waiting to enter
   the critical section. When the wait loop detects that it cannot
   enter the critical section, it calls Sleep, so it blocks instead
   of busy looping.

   If you run this program several times, you will see that most of
   the time the two threads increment their counters in a lopsided
   way, but not as lopsided as the last version. Why?
*/
#include <stdio.h>
#include <windows.h>

// global variables
int runFlag = TRUE;
unsigned int count = 0, count1 = 0, count2 = 0;

// these global variables are used to implement mutual exclusion
int flag1;
int flag2;
int turn;

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   // create threads
   printf("race7me: Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   // let the threads run for a while
   Sleep(2000);
   // now stop the threads
   runFlag = FALSE;
   Sleep(20);

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   //Sleep(0);  // try giving up the CPU right away
   //flag2 = 1; // try pretending that thread2 is in its critical section
   while(runFlag)
   {
      flag1 = 1;
      turn = 1;
      while (turn == 1 && flag2 == 1) Sleep(0);
      // begin critical section
      old_count = count++;
      new_count = count;
      // end critical section
      flag1 = 0;

      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
      count1++;
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      flag2 = 1;
      turn = 2;
      while (turn == 2 && flag1 == 1) Sleep(0);
      // begin critical section
      old_count = count++;
      new_count = count;
      // end critical section
      flag2 = 0;

      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
      count2++;
   }
   return 0;
}//thread2