/*
     This program, like the last two, has two threads each of
     which prints out rows of letters. But in this program, each
     thread prints one row by using a single call to the C library
     function printf(). What is very interesting here is that the
     output that we get depends on which implementation of the C
     library we use.

     On my Windows 2000 machine, the cl and gcc compiler systems
     produce executable versions of this program that have the output
     synchronized in the "right" way, that is, we see blocks of a's in
     rows of 50 and then blocks of b's in rows of 50. With these two
     compilers, there are no mixed up rows with some letter a's and some
     letter b's. Once a thread requests that printf() print out a single
     row, that output request does not get interrupted by the other
     thread's output.

     But the Lcc-Win32 compiler system produces an executable where the
     output is not synchronized.
*/
#include <stdio.h>
#include <windows.h>

// global flag
static int runFlag = TRUE;

int main( int argc, char *argv[])
{
   //thread function prototypes
   DWORD WINAPI thread1(LPVOID);
   DWORD WINAPI thread2(LPVOID);

   // thread function parameters
   int threadParam1 = 0;
   int threadParam2 = 0;

   // create threads
   printf("Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   Sleep(2000);     // let the threads run for a while
   runFlag = FALSE; // now stop the threads
   Sleep(200);      // give them time to stop
   printf("\nThreads stopped.\n");

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   while(runFlag)
   {
      // print a row of 50 letters
      printf("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n");
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   while(runFlag)
   {
      // print a row of 50 letters
      printf("bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb\n");
   }
   return 0;
}//thread2
