/*
     This program demonstrates a very simple example
     of a critical section in a thread. This program
     starts up two threads, both of which print out
     a single letter in rows of 10. In the output we
     expect to see a block of a's, in rows of 10, followed
     by a block of b's, in rows of 10, etc. But instead
     we see that where one block begins and another ends
     a row can become mixed up with a's and b's. That is
     because a thread's time quantum can run out in the
     middle of printing a row. When that happens, one
     thread can start one of its rows in the middle of
     the other thread's row.

     This program shows that we need a way for a thread to
     be able to prevent other threads from interupting it
     at certain "critical times".
*/
#include <stdio.h>    // printf()
#include <unistd.h>   // sleep()
#include <pthread.h>

// global flag
static int runFlag = 1;

int main( int argc, char *argv[])
{
   //thread function prototypes
   void *thread1(void *vargp);
   void *thread2(void *vargp);

   // thread function parameters
   int threadParam1 = 0;
   int threadParam2 = 0;

   pthread_t tid1;
   pthread_t tid2;

   // create threads
   printf("Starting threads\n");
   pthread_create(&tid1, NULL, thread1, &threadParam1);
   pthread_create(&tid2, NULL, thread2, &threadParam2);

   sleep(2);     // let the threads run for a while
   runFlag = 0;  // now stop the threads
   sleep(1);     // give them time to stop
   printf("\nThreads stopped.\n");

   return 0;
}//main


void *thread1(void *vargp)
{
   while(runFlag)
   {
      int i;
      // print a row of 10 letters
      for(i = 0; i < 10; i++)
      {
         printf("a");
      }
      printf("\n");
   }
   return NULL;
}//thread1


void *thread2(void *vargp)
{
   while(runFlag)
   {
      int i;
      // print a row of 10 letters
      for(i = 0; i < 10; i++)
      {
         printf("b");
      }
      printf("\n");
   }
   return NULL;
}//thread2
