/*
     This program creates a thread that changes its characteristic
     over time, form "compute bound" to "I/O bound" to blocked.

     Use this program along with various monitoring programs
     to get an idea of how the operating system schedules threads.
*/
#include <stdio.h>    // printf()
#include <unistd.h>   // sleep()
#include <pthread.h>
#include <math.h>

int main()
{
   // thread function prototype
   void *threadWork(void *vargp);

   pthread_t threadID;

   // create new thread (note that this function call has no error checking)
   pthread_create(&threadID, NULL, threadWork, NULL);
   printf("Created thread with thread ID number %d.\n", (int)threadID);

   while(1) // loop for ever
   {
      sleep(10);  // try commenting out this line
   }

   return 0;
}//main


/*
    This thread function does "parameterized work". The
    thread cycles endlessly between being compute bound,
    sleeping (i.e., being blocked), and being I/O bound.
    The three parameters computeTime, sleepTime, and ioTime
    can be adjusted to control the relative amounts of each
    of these kinds of "work".
*/
void *threadWork(void *vargp)
{
   double y;
   const double x = 3.14159;
   const double e = 2.7183;
   int i;
   const int computeTime = 3000000;
   const int sleepTime = 3;
   const int ioTime = 60000;

   while(1)
   {
      printf("compute\n");
      for(i = 0; i < computeTime; i++)
         y = pow(x,e);

      printf("sleep\n");
      sleep(sleepTime);

      printf("I/O\n");
      for(i = 0; i < ioTime; i++)
         printf("a");
      printf("\n");
   }
   return NULL;
}//threadWork
